package com.epson.epos2_lfcprinter;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.IntentSender;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.annotation.NonNull;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;

import com.epson.epos2.Epos2CallbackCode;
import com.epson.epos2.Log;
import com.epson.epos2.printer.LFCPrinter;
import com.epson.epos2.printer.LFCPrinterStatusInfo;
import com.epson.epos2.printer.LFCSendCompleteListener;
import com.epson.epos2.printer.LFCPrintCompleteListener;

import com.google.android.gms.common.api.CommonStatusCodes;
import com.google.android.gms.common.api.ResolvableApiException;
import com.google.android.gms.location.*;
import com.google.android.gms.tasks.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class MainActivity extends Activity implements View.OnClickListener, LFCSendCompleteListener, LFCPrintCompleteListener {

    private Context mContext = null;
    public EditText mEditTarget = null;
    public static Spinner mSpnSeries = null;
    public Spinner mSpnLang = null;
    public EditText mEditJobNumber = null;
    public static LFCPrinter mLfcPrinter = null;
    public static ProgressIndicator mProgressIndicator = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        mContext = this;

        mProgressIndicator = new ProgressIndicator(mContext);

        int[] target = {
            R.id.btnDiscovery,
            R.id.btnSampleLabel,
        };

        for (int i = 0; i < target.length; i++) {
            Button button = (Button)findViewById(target[i]);
            button.setOnClickListener(this);
        }

        mSpnSeries = (Spinner)findViewById(R.id.spnModel);
        ArrayAdapter<SpnModelsItem> seriesAdapter = new ArrayAdapter<SpnModelsItem>(this, android.R.layout.simple_spinner_item);
        seriesAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        seriesAdapter.add(new SpnModelsItem(getString(R.string.printerseries_l100), LFCPrinter.TM_L100));
        mSpnSeries.setAdapter(seriesAdapter);
        mSpnSeries.setSelection(0);
        mSpnSeries.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                finalizeObject();
                initializeObject();
            }
            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                ;
            }
        });



        mSpnLang = (Spinner)findViewById(R.id.spnLang);
        ArrayAdapter<SpnModelsItem> langAdapter = new ArrayAdapter<SpnModelsItem>(this, android.R.layout.simple_spinner_item);
        langAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        langAdapter.add(new SpnModelsItem(getString(R.string.lang_ank), LFCPrinter.MODEL_ANK));
        langAdapter.add(new SpnModelsItem(getString(R.string.lang_japanese), LFCPrinter.MODEL_JAPANESE));
        langAdapter.add(new SpnModelsItem(getString(R.string.lang_chinese), LFCPrinter.MODEL_CHINESE));
        langAdapter.add(new SpnModelsItem(getString(R.string.lang_taiwan), LFCPrinter.MODEL_TAIWAN));
        langAdapter.add(new SpnModelsItem(getString(R.string.lang_korean), LFCPrinter.MODEL_KOREAN));
        langAdapter.add(new SpnModelsItem(getString(R.string.lang_thai), LFCPrinter.MODEL_THAI));
        langAdapter.add(new SpnModelsItem(getString(R.string.lang_southasia), LFCPrinter.MODEL_SOUTHASIA));
        mSpnLang.setAdapter(langAdapter);
        mSpnLang.setSelection(0);
        mSpnLang.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                finalizeObject();
                initializeObject();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                ;
            }
        });

        mEditTarget = (EditText)findViewById(R.id.edtTarget);
        mEditJobNumber = (EditText)findViewById(R.id.edtJobNumber);

        initializeObject();

        try {
            Log.setLogSettings(mContext, Log.PERIOD_TEMPORARY, Log.OUTPUT_STORAGE, null, 0, 50, Log.LOGLEVEL_LOW);
        }
        catch (Exception e) {
            ShowMsg.showException(e, "setLogSettings", mContext);
        }
    }

    @Override
    protected void onDestroy() {

        finalizeObject();

        super.onDestroy();
    }

    @Override
    protected void onActivityResult(int requestCode, final int resultCode, final Intent data) {
        if (data != null && resultCode == RESULT_OK) {
            String target = data.getStringExtra(getString(R.string.title_target));
            if (target != null) {
                EditText mEdtTarget = (EditText)findViewById(R.id.edtTarget);
                mEdtTarget.setText(target);
            }
        }
    }

    @Override
    public void onClick(View v) {
        Intent intent = null;

        switch (v.getId()) {
            case R.id.btnDiscovery:
                intent = new Intent(this, DiscoveryActivity.class);
                startActivityForResult(intent, 0);
                break;

            case R.id.btnSampleLabel:
                mProgressIndicator.beginProgress(getString(R.string.progress_msg));
                if (!runPrintLabelSequence()) {
                    mProgressIndicator.endProgress();
                }
                break;

            default:
                // Do nothing
                break;
        }
    }

    private boolean runPrintLabelSequence() {

        int jobNumber;
        try {
            jobNumber = Integer.parseInt(mEditJobNumber.getText().toString());
        }catch (Exception e) {
            ShowMsg.showException(e, "", mContext);
            return false;
        }

        if (!createLabelData(jobNumber)) {
            return false;
        }

        if (!printData(jobNumber)) {
            return false;
        }

        updateJobNumber();

        return true;
    }

    private boolean createLabelData(int jobNumber) {
        
        if (mLfcPrinter == null) {
            return false;
        }

        try {
            mLfcPrinter.addTextAlign(LFCPrinter.ALIGN_CENTER);
            mLfcPrinter.addTextSize(2,2);
            mLfcPrinter.addText("THE STORE Delivery\n");
            mLfcPrinter.addTextSize(1,1);
            mLfcPrinter.addText("Order Prepared For\n");
            mLfcPrinter.addTextSize(2,2);
            mLfcPrinter.addText("TEST B\n\n");

            mLfcPrinter.addTextAlign(LFCPrinter.ALIGN_LEFT);
            mLfcPrinter.addTextSize(1,1);
            mLfcPrinter.addText("Ready for delivery : 2/5/2020 4:21:26 PM\n");
            mLfcPrinter.addText(String.format(Locale.US, "jobNumber   : %d\n", jobNumber));
            mLfcPrinter.addText("Total Items : 3\n\n");

            mLfcPrinter.addText("1 THE STORE combo\n");
            mLfcPrinter.addText("    1 OHEIDA 3PK SPRINGF\n");
            mLfcPrinter.addText("    1 CANDYMAKER ASSORT\n");
            mLfcPrinter.addText("    1 REPOSE 4PCPM CHOC\n\n");

            mLfcPrinter.addTextAlign(LFCPrinter.ALIGN_CENTER);
            mLfcPrinter.addText("Questions? Contact THE STORE at\n");
            mLfcPrinter.addText("https://www.*****\n");

            mLfcPrinter.addCut(LFCPrinter.CUT_FEED);
        }
        catch (Exception e) {
            mLfcPrinter.clearCommandBuffer();
            return false;
        }

        return true;
    }

    private boolean printData(int jobNumber) {
        if (mLfcPrinter == null) {
            return false;
        }

        if (!connectLFCPrinter()) {
            mLfcPrinter.clearCommandBuffer();
            return false;
        }

        try {
            mLfcPrinter.sendLFCData(LFCPrinter.PARAM_DEFAULT, jobNumber);
        }
        catch (Exception e) {
            ShowMsg.showException(e, "sendLFCData", mContext);
            mLfcPrinter.clearCommandBuffer();
            disconnectLFCPrinter();
            return false;
        }

        mLfcPrinter.clearCommandBuffer();
        return true;
    }

    private void initializeObject() {
        try {
            mLfcPrinter = new LFCPrinter(((SpnModelsItem) mSpnSeries.getSelectedItem()).getModelConstant(),((SpnModelsItem) mSpnLang.getSelectedItem()).getModelConstant(),
                    mContext);
        }
        catch (Exception e) {
            ShowMsg.showException(e, "LFCPrinter", mContext);
            return;
        }

        mLfcPrinter.setSendCompleteEventListener(this);
        mLfcPrinter.setPrintCompleteEventListener(this);
    }

    private void finalizeObject() {
        if (mLfcPrinter == null) {
            return;
        }

        mLfcPrinter.setPrintCompleteEventListener(null);
        mLfcPrinter.setSendCompleteEventListener(null);

        mLfcPrinter = null;
    }

    private boolean connectLFCPrinter() {
        if (mLfcPrinter == null) {
            return false;
        }

        try {
            mLfcPrinter.connect(mEditTarget.getText().toString(), LFCPrinter.PARAM_DEFAULT);
        }
        catch (Exception e) {
            ShowMsg.showException(e, "connect", mContext);
            return false;
        }

        return true;
    }

    private void disconnectLFCPrinter() {
        if (mLfcPrinter == null) {
            return;
        }

        try {
            mLfcPrinter.disconnect();
        } catch (Exception e) {
            ShowMsg.showException(e, "disconnect", mContext);
        }
    }

    private String makeErrorMessage(LFCPrinterStatusInfo status) {
        String msg = "";

        if (status.getOnline() == LFCPrinter.FALSE) {
            msg += getString(R.string.handlingmsg_err_offline);
        }
        if (status.getConnection() == LFCPrinter.FALSE) {
            msg += getString(R.string.handlingmsg_err_no_response);
        }
        if (status.getCoverOpen() == LFCPrinter.TRUE) {
            msg += getString(R.string.handlingmsg_err_cover_open);
        }
        if (status.getPaper() == LFCPrinter.PAPER_EMPTY) {
            msg += getString(R.string.handlingmsg_err_receipt_end);
        }
        if (status.getPaperFeed() == LFCPrinter.TRUE || status.getPanelSwitch() == LFCPrinter.SWITCH_ON) {
            msg += getString(R.string.handlingmsg_err_paper_feed);
        }
        if (status.getErrorStatus() == LFCPrinter.MECHANICAL_ERR || status.getErrorStatus() == LFCPrinter.AUTOCUTTER_ERR) {
            msg += getString(R.string.handlingmsg_err_autocutter);
            msg += getString(R.string.handlingmsg_err_need_recover);
        }
        if (status.getErrorStatus() == LFCPrinter.UNRECOVER_ERR) {
            msg += getString(R.string.handlingmsg_err_unrecover);
        }
        if (status.getErrorStatus() == LFCPrinter.AUTORECOVER_ERR) {
            if (status.getAutoRecoverError() == LFCPrinter.HEAD_OVERHEAT) {
                msg += getString(R.string.handlingmsg_err_overheat);
                msg += getString(R.string.handlingmsg_err_head);
            }
            if (status.getAutoRecoverError() == LFCPrinter.MOTOR_OVERHEAT) {
                msg += getString(R.string.handlingmsg_err_overheat);
                msg += getString(R.string.handlingmsg_err_motor);
            }
            if (status.getAutoRecoverError() == LFCPrinter.BATTERY_OVERHEAT) {
                msg += getString(R.string.handlingmsg_err_overheat);
                msg += getString(R.string.handlingmsg_err_battery);
            }
            if (status.getAutoRecoverError() == LFCPrinter.WRONG_PAPER) {
                msg += getString(R.string.handlingmsg_err_wrong_paper);
            }
        }
        if (status.getBatteryLevel() == LFCPrinter.BATTERY_LEVEL_0) {
            msg += getString(R.string.handlingmsg_err_battery_real_end);
        }
        if (status.getRemovalWaiting() == LFCPrinter.REMOVAL_WAIT_PAPER) {
            msg += getString(R.string.handlingmsg_removalWait);
        }

        return msg;
    }

    private void updateJobNumber() {
        try {
            int jobNumber = Integer.parseInt(mEditJobNumber.getText().toString());
            jobNumber++;
            String jobNumberStr = String.valueOf(jobNumber);
            mEditJobNumber.setText(jobNumberStr);
        } catch (Exception e) {
            ShowMsg.showException(e,"",mContext);
        }
    }

    private void dispLogs(String log) {
        EditText edtLogs = (EditText)findViewById(R.id.edtLog);

        if(log == null) {
            return;
        }
        edtLogs.append(log);
    }

    @Override
    public void onSendComplete(final LFCPrinter lfcPrinterObj, final int jobNumber, final int code, final LFCPrinterStatusInfo status) {
        runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                String msg = "[onSendComplete]";
                msg += "code : " + ShowMsg.getCodeText(code) + ", " + "jobNumber : " + jobNumber + ", " + "status : " + makeErrorMessage(status) + "\n";
                dispLogs(msg);

                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        disconnectLFCPrinter();
                    }
                }).start();

                mProgressIndicator.endProgress();
            }
        });
    }

    @Override
    public void onPrintComplete(LFCPrinter lfcPrinterObj, final int jobNumber) {
        runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                String msg = "[onPrintComplete]";
                msg += "jobNumber : " + jobNumber + "\n";
                dispLogs(msg);
            }
        });
    }

}
