package com.epson.epos2_linedisplay;

import android.content.Context;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ToggleButton;

import com.epson.epos2.Epos2Exception;
import com.epson.epos2.linedisplay.LineDisplay;
import com.epson.epos2.linedisplay.ReceiveListener;


public class D30Fragment extends Fragment implements View.OnClickListener, ReceiveListener  {
    private EditText mEditTarget = null;
    private ToggleButton mToggleBlink = null;
    private EditText mEditData = null;
    private Button mBtnDisplay = null;
    private Context mContext = null;
    private LineDisplay mLineDisplay = null;

    public static D30Fragment newInstance() {
        return new D30Fragment();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        mContext = getContext();
        View rootView = inflater.inflate(R.layout.fragment_d30, container, false);
        mEditTarget = rootView.findViewById(R.id.editTarget);
        mToggleBlink = rootView.findViewById(R.id.toggleBlink);
        mEditData = rootView.findViewById(R.id.editData);
        mBtnDisplay = rootView.findViewById(R.id.btnDisplay);
        mBtnDisplay.setOnClickListener(this);

        initializeObject();
        return rootView;
    }

    @Override
    public void onDestroyView(){
        finalizeObject();
        super.onDestroyView();
        return;
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnDisplay:
                updateButtonState(false);
                if (!runLineDisplaySequence()) {
                    updateButtonState(true);
                }
                break;

            default:
                // Do nothing
                break;
        }
    }

    private boolean runLineDisplaySequence() {

        if (!createDisplayData()) {
            return false;
        }

        if (!connectDisplay()) {
            return false;
        }

        try {
            mLineDisplay.sendData();
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "sendData", mContext);
            disconnectDisplay();
            return false;
        }

        return true;
    }

    private boolean initializeObject() {
        try {
            mLineDisplay = new LineDisplay(LineDisplay.DM_D30, mContext);
        }
        catch (Exception e) {
            ShowMsg.showException(e, "LineDisplay", mContext);

            return false;
        }

        mLineDisplay.setReceiveEventListener(this);

        return true;
    }

    private void finalizeObject() {
        if (mLineDisplay == null) {
            return;
        }

        mLineDisplay.setReceiveEventListener(null);

        mLineDisplay = null;
    }

    private boolean connectDisplay() {
        if (mLineDisplay == null) {
            return false;
        }

        try {
            mLineDisplay.connect(mEditTarget.getText().toString(), LineDisplay.PARAM_DEFAULT);
        }
        catch (Epos2Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "connect", mContext);

            return false;
        }

        return true;
    }

    private void disconnectDisplay() {
        if (mLineDisplay == null) {
            return;
        }

        try {
            mLineDisplay.disconnect();
        }
        catch (final Exception e) {
            getActivity().runOnUiThread(new Runnable() {
                @Override
                public synchronized void run() {
                    ShowMsg.showException(e, "disconnect", mContext);
                }
            });
        }

        mLineDisplay.clearCommandBuffer();
    }

    private boolean createDisplayData() {
        String method = "";

        if (mLineDisplay == null) {
            return false;
        }

        try {
            method = "addInitialize";
            mLineDisplay.addInitialize();

            method = "addSetCursorPosition";
            mLineDisplay.addSetCursorPosition(1, 1);

            if (mToggleBlink.isChecked()) {
                method = "addSetBlink";
                mLineDisplay.addSetBlink(1000);
            }

            method = "addText";
            mLineDisplay.addText(mEditData.getText().toString());
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, method, mContext);
            return false;
        }

        return true;
    }

    private void updateButtonState(boolean state) {
        mBtnDisplay.setEnabled(state);
    }

    @Override
    public void onDispReceive(final LineDisplay displayObj, final int code) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                ShowMsg.showResult(code, mContext);

                updateButtonState(true);

                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        disconnectDisplay();
                    }
                }).start();
            }
        });
    }
}
