package com.epson.epos2_linedisplay;

import android.content.Context;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import java.io.ByteArrayOutputStream;

import com.epson.epos2.Epos2Exception;
import com.epson.epos2.linedisplay.LineDisplay;
import com.epson.epos2.linedisplay.ReceiveListener;

public class D70Fragment extends Fragment implements View.OnClickListener, ReceiveListener  {
    private EditText mEditText = null;
    private Button mBtnLandscape = null;
    private Button mBtnPortrait = null;
    private Button mBtnRegisterImg = null;
    private Button mBtnOrder = null;
    private Button mBtnCheck = null;
    private Button mBtnStart = null;
    private Button mBtnStop = null;
    private Context mContext = null;
    private LineDisplay mLineDisplay = null;
    private byte[] pngBinaryLandscape = null;
    private byte[] pngBinaryPortrait = null;
    private boolean bIsWorkingSlideshow = false;
    private boolean bIsLandscape = false;

    public static D70Fragment newInstance() {
        return new D70Fragment();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        mContext = getContext();
        View rootView = inflater.inflate(R.layout.fragment_d70, container, false);
        mEditText = rootView.findViewById(R.id.editTarget);
        mBtnLandscape = rootView.findViewById(R.id.btnLandscape);
        mBtnPortrait = rootView.findViewById(R.id.btnPortrait);
        mBtnRegisterImg = rootView.findViewById(R.id.btnRegisterImg);
        mBtnOrder = rootView.findViewById(R.id.btnOrder);
        mBtnCheck = rootView.findViewById(R.id.btnCheck);
        mBtnStart = rootView.findViewById(R.id.btnStart);
        mBtnStop = rootView.findViewById(R.id.btnStop);
        mBtnLandscape.setOnClickListener(this);
        mBtnPortrait.setOnClickListener(this);
        mBtnRegisterImg.setOnClickListener(this);
        mBtnOrder.setOnClickListener(this);
        mBtnCheck.setOnClickListener(this);
        mBtnStart.setOnClickListener(this);
        mBtnStop.setOnClickListener(this);
        mBtnOrder.setEnabled(false);
        mBtnCheck.setEnabled(false);

        initializeObject();
        return rootView;
    }

    @Override
    public void onDestroyView(){
        finalizeObject();
        super.onDestroyView();
        return;
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnLandscape:
                landscapeScreen();
                break;
            case R.id.btnPortrait:
                portraitScreen();
                break;
            case R.id.btnRegisterImg:
                registerImage();
                break;
            case R.id.btnOrder:
                orderDemo();
                break;
            case R.id.btnCheck:
                checkDemo();
                break;
            case R.id.btnStart:
                startSlideShow();
                break;
            case R.id.btnStop:
                stopSlideShow();
                break;
            default:
                // Do nothing
                break;
        }
    }

    private  void landscapeScreen(){
        bIsLandscape = true;
        if (mLineDisplay == null) {
            return;
        }

        try{
            mLineDisplay.addInitialize();
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addInitialize", mContext);
            return;
        }

        try{
            mLineDisplay.addCreateScreenCustom(LineDisplay.LANDSCAPE_LAYOUT_MODE_5, 44, 13);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addCreateScreenCustom", mContext);
            return;
        }

        try{
            mLineDisplay.addCreateTextArea(1, 1, 1, 17, 5, LineDisplay.PARAM_DEFAULT);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addCreateTextArea", mContext);
            return;
        }

        try{
            mLineDisplay.addBackgroundColor(LineDisplay.ALL_ROWS, 255,255,255);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addBackgroundColor", mContext);
            return;
        }

        try{
            mLineDisplay.addCreateTextArea(2, 18, 1, 16, 5, LineDisplay.PARAM_DEFAULT);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addCreateTextArea", mContext);
            return;
        }

        try{
            mLineDisplay.addBackgroundColor(LineDisplay.ALL_ROWS, 0,255,255);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addBackgroundColor", mContext);
            return;
        }

        try{
            mLineDisplay.addStartSlideShow(500);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addStartSlideShow", mContext);
            return;
        }

        updateButtonState(false);

        if (!connectDisplay()) {
            updateButtonState(true);
            return;
        }

        try {
            mLineDisplay.sendData();
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "sendData", mContext);
            disconnectDisplay();
            updateButtonState(true);
            return;
        }
        return;
    }

    private  void portraitScreen(){
        bIsLandscape = false;
        if (mLineDisplay == null) {
            return;
        }

        try{
            mLineDisplay.addInitialize();
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addInitialize", mContext);
            return;
        }

        try{
            mLineDisplay.addCreateScreenCustom(LineDisplay.PORTRAIT_LAYOUT_MODE_3, 22, 19);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addCreateScreenCustom", mContext);
            return;
        }

        try{
            mLineDisplay.addCreateTextArea(1, 1, 1, 20, 3, LineDisplay.PARAM_DEFAULT);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addCreateTextArea", mContext);
            return;
        }

        try{
            mLineDisplay.addBackgroundColor(LineDisplay.ALL_ROWS, 0,255,255);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addBackgroundColor", mContext);
            return;
        }

        try{
            mLineDisplay.addCreateTextArea(2, 1, 4, 20, 5, LineDisplay.PARAM_DEFAULT);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addCreateTextArea", mContext);
            return;
        }

        try{
            mLineDisplay.addBackgroundColor(LineDisplay.ALL_ROWS, 255,255,255);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addBackgroundColor", mContext);
            return;
        }

        try{
            mLineDisplay.addStartSlideShow(500);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addStartSlideShow", mContext);
            return;
        }

        updateButtonState(false);
        if (!connectDisplay()) {
            updateButtonState(true);
            return;
        }

        try {
            mLineDisplay.sendData();
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "sendData", mContext);
            disconnectDisplay();
            updateButtonState(true);
            return;
        }

        return;
    }

    private  void registerImage(){
        if (mLineDisplay == null) {
            return;
        }

        Bitmap pngDataLandscape = BitmapFactory.decodeResource(getResources(), R.drawable.thanke800);
        ByteArrayOutputStream streamLandscape = new ByteArrayOutputStream();
        pngDataLandscape.compress(Bitmap.CompressFormat.PNG, 100, streamLandscape);
        pngBinaryLandscape = streamLandscape.toByteArray();

        try{
            mLineDisplay.addRegisterDownloadImage(pngBinaryLandscape, 65,65);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addRegisterDownloadImage", mContext);
            return;
        }

        Bitmap pngDataPortrait = BitmapFactory.decodeResource(getResources(), R.drawable.thanke);
        ByteArrayOutputStream streamPortrait = new ByteArrayOutputStream();
        pngDataPortrait.compress(Bitmap.CompressFormat.PNG, 100, streamPortrait);
        pngBinaryPortrait = streamPortrait.toByteArray();

        try{
            mLineDisplay.addRegisterDownloadImage(pngBinaryPortrait, 66,66);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addRegisterDownloadImage", mContext);
            return;
        }


        updateButtonState(false);
        if (!connectDisplay()) {
            updateButtonState(true);
            return;
        }

        try {
            mLineDisplay.sendData();
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "sendData", mContext);
            disconnectDisplay();
            updateButtonState(true);
            return;
        }

        return;
    }

    private  void orderDemo(){

        String text = "";
        if (mLineDisplay == null) {
            return;
        }

        try{
            mLineDisplay.addClearSymbol();
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addClearSymbol", mContext);
            return;
        }

        if(bIsWorkingSlideshow == false) {
            try{
                mLineDisplay.addClearImage();
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addClearImage", mContext);
                return;
            }

            try{
                mLineDisplay.addStartSlideShow(500);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addStartSlideShow", mContext);
                return;
            }
        }


        if(bIsLandscape) {
            try{
                mLineDisplay.addSetCurrentTextArea(1);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSetCurrentTextArea", mContext);
                return;
            }

            text += "Item:\n";
            text += "  Cut & Sewn\n";
            text += "U.PC.45.00->";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            text = "22.50";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 255,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            text = "QTY:            3";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            try{
                mLineDisplay.addSetCurrentTextArea(2);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSetCurrentTextArea", mContext);
                return;
            }


            text = "SubTotal:\n";
            text += " $67.50\n";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }
        }else
        {
            try{
                mLineDisplay.addSetCurrentTextArea(1);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSetCurrentTextArea", mContext);
                return;
            }

            text = "Item:\n";
            text += "   Cut & Sewn\n";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            try{
                mLineDisplay.addSetCurrentTextArea(2);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSetCurrentTextArea", mContext);
                return;
            }

            text = "Qty:               3";
            text += "U.PC.:        $45.00";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            text = "             (22.50)\n";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 255,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            text = "SubTotal:     $67.50";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }
        }
        updateButtonState(false);
        if (!connectDisplay()) {
            updateButtonState(true);
            return;
        }

        try {
            mLineDisplay.sendData();
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "sendData", mContext);
            disconnectDisplay();
            updateButtonState(true);
            return;
        }
        return;
    }

    private  void checkDemo(){
        String text = "";
        if (mLineDisplay == null) {
            return;
        }

        try{
            mLineDisplay.addStopSlideShow();
        }catch (Exception e) {
            ShowMsg.showException(e, "addStopSlideShow", mContext);
            return;
        }

        if(bIsLandscape) {
            try{
                mLineDisplay.addDownloadImage(65,65, 0,0,800,240);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addDownloadImage", mContext);
                return;
            }

            try{
                mLineDisplay.addSetCurrentTextArea(1);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSetCurrentTextArea", mContext);
                return;
            }

            try{
                mLineDisplay.addClearCurrentTextArea();
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addClearCurrentTextArea", mContext);
                return;
            }

            text = "Store Information\n";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            try{
                mLineDisplay.addSetCurrentTextArea(2);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSetCurrentTextArea", mContext);
                return;
            }

            try{
                mLineDisplay.addClearCurrentTextArea();
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addClearCurrentTextArea", mContext);
                return;
            }

            text = "Total:    $66.50";
            text += "Cash:    $100.50";
            text += "Change:   $34.00";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            try{
                mLineDisplay.addSymbol("https://www.epson-pos.com/", LineDisplay.SYMBOL_QRCODE_MODEL_2, LineDisplay.LEVEL_L, 4, 4,120, 70, LineDisplay.FALSE);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSymbol", mContext);
                return;
            }
        }
        else {
            try{
                mLineDisplay.addDownloadImage(66,66, 0,0,480,400);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addDownloadImage", mContext);
                return;
            }


            try{
                mLineDisplay.addSetCurrentTextArea(1);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSetCurrentTextArea", mContext);
                return;
            }

            try{
                mLineDisplay.addClearCurrentTextArea();
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addClearCurrentTextArea", mContext);
                return;
            }

            text = "Total:        $67.50";
            text += "Cash:        $100.00";
            text +="Change:       $32.50";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            try{
                mLineDisplay.addSetCurrentTextArea(2);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSetCurrentTextArea", mContext);
                return;
            }

            try{
                mLineDisplay.addClearCurrentTextArea();
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addClearCurrentTextArea", mContext);
                return;
            }

            text = "\n Store Information\n";
            try{
                mLineDisplay.addText(text, LineDisplay.PARAM_UNUSE, LineDisplay.PARAM_UNUSE, LineDisplay.LANG_EN, 0,0,0);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addText", mContext);
                return;
            }

            try{
                mLineDisplay.addSymbol("https://www.epson-pos.com/", LineDisplay.SYMBOL_QRCODE_MODEL_2, LineDisplay.LEVEL_L, 4, 4,160, 260, LineDisplay.FALSE);
            }catch (Exception e) {
                mLineDisplay.clearCommandBuffer();
                ShowMsg.showException(e, "addSymbol", mContext);
                return;
            }
        }
        updateButtonState(false);
        if (!connectDisplay()) {
            updateButtonState(true);
            return;
        }

        try {
            mLineDisplay.sendData();
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "sendData", mContext);
            disconnectDisplay();
            updateButtonState(true);
            return;
        }
        return;
    }

    private  void startSlideShow(){
        bIsWorkingSlideshow = true;
        if (mLineDisplay == null) {
            return;
        }
        try{
            mLineDisplay.addInitialize();
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addInitialize", mContext);
            return;
        }

        try{
            mLineDisplay.addStartSlideShow(500);
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addStartSlideShow", mContext);
            return;
        }

        updateButtonState(false);
        if (!connectDisplay()) {
            updateButtonState(true);
            return;
        }

        try {
            mLineDisplay.sendData();
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "sendData", mContext);
            disconnectDisplay();
            updateButtonState(true);
            return;
        }

        return;
    }

    private  void stopSlideShow(){

        if (mLineDisplay == null) {
            return;
        }

        try{
            mLineDisplay.addInitialize();
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addInitialize", mContext);
            return;
        }

        try{
            mLineDisplay.addStopSlideShow();
        }catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "addStopSlideShow", mContext);
            return;
        }

        bIsWorkingSlideshow = false;
        updateButtonState(false);
        if (!connectDisplay()) {
            updateButtonState(true);
            return;
        }

        try {
            mLineDisplay.sendData();
        }
        catch (Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "sendData", mContext);
            disconnectDisplay();
            updateButtonState(true);
            return;
        }
        return;
    }

    private boolean initializeObject() {
        try {
            mLineDisplay = new LineDisplay(LineDisplay.DM_D70, mContext);
        }
        catch (Exception e) {
            ShowMsg.showException(e, "LineDisplay", mContext);

            return false;
        }

        mLineDisplay.setReceiveEventListener(this);

        return true;
    }

    private void finalizeObject() {
        if (mLineDisplay == null) {
            return;
        }

        mLineDisplay.setReceiveEventListener(null);

        mLineDisplay = null;
    }

    private boolean connectDisplay() {
        if (mLineDisplay == null) {
            return false;
        }

        try {
            mLineDisplay.connect(mEditText.getText().toString(), LineDisplay.PARAM_DEFAULT);
        }
        catch (Epos2Exception e) {
            mLineDisplay.clearCommandBuffer();
            ShowMsg.showException(e, "connect", mContext);

            return false;
        }

        return true;
    }

    private void disconnectDisplay() {
        if (mLineDisplay == null) {
            return;
        }

        try {
            mLineDisplay.disconnect();
        }
        catch (final Exception e) {
            getActivity().runOnUiThread(new Runnable() {
                @Override
                public synchronized void run() {
                    ShowMsg.showException(e, "disconnect", mContext);
                }
            });
        }

        mLineDisplay.clearCommandBuffer();
    }

    private void updateButtonState(boolean state) {
        mBtnLandscape.setEnabled(state);
        mBtnPortrait.setEnabled(state);
        mBtnOrder.setEnabled(state);
        mBtnCheck.setEnabled(state);
        mBtnStart.setEnabled(state);
        mBtnStop.setEnabled(state);
    }

    @Override
    public void onDispReceive(final LineDisplay displayObj, final int code) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                ShowMsg.showResult(code, mContext);
                updateButtonState(true);
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        disconnectDisplay();
                    }
                }).start();
            }
        });
    }
}
